@@ -368,7 +368,8 @@ typedef int ssl_algo_t;
 #define SSL_OPT_EXPORTCERTDATA (1<<2)
 #define SSL_OPT_FAKEBASICAUTH  (1<<3)
 #define SSL_OPT_STRICTREQUIRE  (1<<4)
-#define SSL_OPT_ALL            (SSL_OPT_COMPATENVVAR|SSL_OPT_EXPORTCERTDATA|SSL_OPT_FAKEBASICAUTH|SSL_OPT_STRICTREQUIRE)
+#define SSL_OPT_OPTRENEGOTIATE (1<<5)
+#define SSL_OPT_ALL            (SSL_OPT_COMPATENVVAR|SSL_OPT_EXPORTCERTDATA|SSL_OPT_FAKEBASICAUTH|SSL_OPT_STRICTREQUIRE|SSL_OPT_OPTRENEGOTIATE)
 typedef int ssl_opt_t;
 
 /*
@@ -638,17 +639,20 @@ void         ssl_scache_store(server_rec *, SSL_SESSION *, int);
 SSL_SESSION *ssl_scache_retrieve(server_rec *, UCHAR *, int);
 void         ssl_scache_remove(server_rec *, SSL_SESSION *);
 void         ssl_scache_expire(server_rec *, time_t);
+void         ssl_scache_status(server_rec *, pool *, void (*)(char *, void *), void *);
 char        *ssl_scache_id2sz(UCHAR *, int);
 void         ssl_scache_dbm_init(server_rec *, pool *);
 void         ssl_scache_dbm_store(server_rec *, ssl_scinfo_t *);
 void         ssl_scache_dbm_retrieve(server_rec *, ssl_scinfo_t *);
 void         ssl_scache_dbm_remove(server_rec *, ssl_scinfo_t *);
 void         ssl_scache_dbm_expire(server_rec *, time_t);
+void         ssl_scache_dbm_status(server_rec *, pool *, void (*)(char *, void *), void *);
 void         ssl_scache_shm_init(server_rec *, pool *);
 void         ssl_scache_shm_store(server_rec *, ssl_scinfo_t *);
 void         ssl_scache_shm_retrieve(server_rec *, ssl_scinfo_t *);
 void         ssl_scache_shm_remove(server_rec *, ssl_scinfo_t *);
 void         ssl_scache_shm_expire(server_rec *, time_t);
+void         ssl_scache_shm_status(server_rec *, pool *, void (*)(char *, void *), void *);
 
 /*  Pass Phrase Support  */
 void         ssl_pphrase_Handle(server_rec *, pool *);
