@@ -76,12 +76,15 @@ static void  ssl_ext_mr_register(void);
 static void  ssl_ext_mr_unregister(void);
 static void  ssl_ext_mp_register(void);
 static void  ssl_ext_mp_unregister(void);
+static void  ssl_ext_ms_register(void);
+static void  ssl_ext_ms_unregister(void);
 
 void ssl_ext_register(void)
 {
     ssl_ext_mlc_register();
     ssl_ext_mr_register();
     ssl_ext_mp_register();
+    ssl_ext_ms_register();
     return;
 }
 
@@ -90,6 +93,7 @@ void ssl_ext_unregister(void)
     ssl_ext_mlc_unregister();
     ssl_ext_mr_unregister();
     ssl_ext_mp_unregister();
+    ssl_ext_ms_unregister();
     return;
 }
 
@@ -360,3 +364,46 @@ static int ssl_ext_mp_write_host_header(
     return DECLINED;
 }
 
+
+/*  _________________________________________________________________
+**
+**  SSL Extension to mod_status
+**  _________________________________________________________________
+*/
+
+static void ssl_ext_ms_display(request_rec *, int, int);
+
+static void ssl_ext_ms_register(void)
+{
+    ap_hook_register("ap::mod_status::display", ssl_ext_ms_display, AP_HOOK_NOCTX);
+    return;
+}
+
+static void ssl_ext_ms_unregister(void)
+{
+    ap_hook_unregister("ap::mod_status::display", ssl_ext_ms_display);
+    return;
+}
+
+static void ssl_ext_ms_display_cb(char *str, void *_r)
+{
+    request_rec *r = (request_rec *)_r;
+    if (str != NULL)
+        ap_rputs(str, r);
+    return;
+}
+
+static void ssl_ext_ms_display(request_rec *r, int no_table_report, int short_report)
+{
+    ap_rputs("<hr>\n", r);
+    ap_rputs("<table cellspacing=0 cellpadding=0>\n", r);
+    ap_rputs("<tr><td bgcolor=\"#000000\">\n", r);
+    ap_rputs("<b><font color=\"#ffffff\" face=\"Arial,Helvetica\">SSL/TLS Session Cache Status:</font></b>\r", r);
+    ap_rputs("</td></tr>\n", r);
+    ap_rputs("<tr><td bgcolor=\"#ffffff\">\n", r);
+    ssl_scache_status(r->server, r->pool, ssl_ext_ms_display_cb, r);
+    ap_rputs("</td></tr>\n", r);
+    ap_rputs("</table>\n", r);
+    return;
+}
+
