@@ -130,6 +130,7 @@ void ssl_hook_NewConnection(conn_rec *conn)
     server_rec *srvr;
     BUFF *fb;
     SSLSrvConfigRec *sc;
+    ap_ctx *apctx;
     SSL *ssl;
     char *cp;
     char *cpVHostID;
@@ -184,8 +185,11 @@ void ssl_hook_NewConnection(conn_rec *conn)
     }
     SSL_clear(ssl);
     SSL_set_session_id_context(ssl, cpVHostID, strlen(cpVHostID));
-    SSL_set_app_data(ssl, conn);  /* conn_rec    (available now)     */
-    SSL_set_app_data2(ssl, NULL); /* request_rec (available later)   */
+    SSL_set_app_data(ssl, conn);
+    apctx = ap_ctx_new(conn->pool);
+    ap_ctx_set(apctx, "ssl::request_rec", NULL);
+    ap_ctx_set(apctx, "ssl::verify::depth", AP_CTX_NUM2PTR(0));
+    SSL_set_app_data2(ssl, apctx);
     SSL_set_fd(ssl, fb->fd);
     ap_ctx_set(fb->ctx, "ssl", ssl);
 
@@ -200,10 +204,10 @@ void ssl_hook_NewConnection(conn_rec *conn)
     }
 
     /*
-	 * Configure the server certificate and private key which should be used
-	 * for this connection. Under combined RSA+DSA situation there can be two
-	 * certs for a server. The SSL library automatically handles this
-	 * internally.
+     * Configure the server certificate and private key which should be used
+     * for this connection. Under combined RSA+DSA situation there can be two
+     * certs for a server. The SSL library automatically handles this
+     * internally.
      */
     if (sc->pPublicCert[SSL_AIDX_RSA] != NULL) {
         if (SSL_use_certificate(ssl, sc->pPublicCert[SSL_AIDX_RSA]) <= 0) {
@@ -341,7 +345,7 @@ void ssl_hook_NewConnection(conn_rec *conn)
             else if (SSL_get_error(ssl, rc) == SSL_ERROR_SYSCALL) {
                 if (errno == EINTR)
                     continue;
-                ssl_log(srvr, SSL_LOG_ERROR|SSL_ADD_ERRNO,
+                ssl_log(srvr, SSL_LOG_ERROR|SSL_ADD_SSLERR|SSL_ADD_ERRNO,
                         "SSL handshake interrupted by system");
                 SSL_set_shutdown(ssl, SSL_RECEIVED_SHUTDOWN);
                 SSL_smart_shutdown(ssl);
@@ -545,14 +549,17 @@ void ssl_hook_CloseConnection(conn_rec *conn)
 int ssl_hook_ReadReq(request_rec *r)
 {
     SSL *ssl;
+    ap_ctx *apctx;
 
     /*
      * Get the SSL connection structure and perform the
      * delayed interlinking from SSL back to request_rec
      */
     ssl = ap_ctx_get(r->connection->client->ctx, "ssl");
-    if (ssl != NULL)
-        SSL_set_app_data2(ssl, r);
+    if (ssl != NULL) {
+        apctx = SSL_get_app_data2(ssl);
+        ap_ctx_set(apctx, "ssl::request_rec", r);
+    }
 
     /*
      * Force the mod_ssl content handler when URL indicates this
@@ -652,18 +659,27 @@ int ssl_hook_Access(request_rec *r)
     char *cp;
     int ok;
     int i;
-    BOOL renegotiate; 
+    BOOL renegotiate;
+    BOOL renegotiate_quick;
 #ifdef SSL_EXPERIMENTAL
     BOOL reconfigured_locations;
     STACK_OF(X509_NAME) *skCAList;
     char *cpCAPath;
     char *cpCAFile;
 #endif
+    X509 *cert;
+    STACK_OF(X509) *certstack;
+    X509_STORE *certstore;
+    X509_STORE_CTX certstorectx;
+    int depth;
     STACK_OF(SSL_CIPHER)  *skCipherOld;
     STACK_OF(SSL_CIPHER)  *skCipher;
+    SSL_CIPHER *pCipher;
+    ap_ctx *apctx;
     int nVerifyOld;
     int nVerify;
     int n;
+    void *vp;
     int rc;
 
     dc  = myDirConfig(r);
@@ -691,83 +707,154 @@ int ssl_hook_Access(request_rec *r)
         return DECLINED;
 
     /*
-     * Support for per-directory SSL connection parameters.  
-     * 
+     * Support for per-directory reconfigured SSL connection parameters.
+     *
      * This is implemented by forcing an SSL renegotiation with the
      * reconfigured parameter suite. But Apache's internal API processing
-     * makes our life very hard, because when internal sub-requests occur we
-     * nevertheless should avoid multiple unnecessary SSL handshakes (they
-     * need network I/O and time to perform). But the optimization for
-     * filtering out the unnecessary handshakes isn't such obvious.
-     * Especially because while Apache is in its sub-request processing the
-     * client could force additional handshakes, too. And these take place
-     * perhaps without our notice. So the only possibility is to ask
-     * OpenSSL whether the renegotiation has to be performed or not. It
-     * has to performed when some parameters which were previously known (by
-     * us) are not those we've now reconfigured (as known by OpenSSL).
+     * makes our life very hard here, because when internal sub-requests occur
+     * we nevertheless should avoid multiple unnecessary SSL handshakes (they
+     * require extra network I/O and especially time to perform). 
+     * 
+     * But the optimization for filtering out the unnecessary handshakes isn't
+     * obvious and trivial.  Especially because while Apache is in its
+     * sub-request processing the client could force additional handshakes,
+     * too. And these take place perhaps without our notice. So the only
+     * possibility is to explicitly _ask_ OpenSSL whether the renegotiation
+     * has to be performed or not. It has to performed when some parameters
+     * which were previously known (by us) are not those we've now
+     * reconfigured (as known by OpenSSL) or (in optimized way) at least when
+     * the reconfigured parameter suite is stronger (more restrictions) than
+     * the currently active one.
      */
-    renegotiate = FALSE;
+    renegotiate            = FALSE;
+    renegotiate_quick      = FALSE;
 #ifdef SSL_EXPERIMENTAL
     reconfigured_locations = FALSE;
 #endif
 
-    /* 
-     * override of SSLCipherSuite
+    /*
+     * Override of SSLCipherSuite
+     *
+     * We provide two options here:
+     *
+     * o The paranoid and default approach where we force a renegotiation when
+     *   the cipher suite changed in _any_ way (which is straight-forward but
+     *   often forces renegotiations too often and is perhaps not what the
+     *   user actually wanted).
+     *
+     * o The optimized and still secure way where we force a renegotiation
+     *   only if the currently active cipher is no longer contained in the
+     *   reconfigured/new cipher suite. Any other changes are not important
+     *   because it's the servers choice to select a cipher from the ones the
+     *   client supports. So as long as the current cipher is still in the new
+     *   cipher suite we're happy. Because we can assume we would have
+     *   selected it again even when other (better) ciphers exists now in the
+     *   new cipher suite. This approach is fine because the user explicitly
+     *   has to enable this via ``SSLOptions +OptRenegotiate''. So we do no
+     *   implicit optimizations.
      */
     if (dc->szCipherSuite != NULL) {
-        /* remember old cipher suite for comparison */
-        if ((skCipherOld = SSL_get_ciphers(ssl)) != NULL)
-            skCipherOld = sk_SSL_CIPHER_dup(skCipherOld);
-        /* configure new cipher suite */
+        /* remember old state */
+        pCipher = NULL;
+        skCipherOld = NULL;
+        if (dc->nOptions & SSL_OPT_OPTRENEGOTIATE)
+            pCipher = SSL_get_current_cipher(ssl);
+        else {
+            skCipherOld = SSL_get_ciphers(ssl);
+            if (skCipherOld != NULL)
+                skCipherOld = sk_SSL_CIPHER_dup(skCipherOld);
+        }
+        /* configure new state */
         if (!SSL_set_cipher_list(ssl, dc->szCipherSuite)) {
             ssl_log(r->server, SSL_LOG_WARN|SSL_ADD_SSLERR,
                     "Unable to reconfigure (per-directory) permitted SSL ciphers");
+            if (skCipherOld != NULL)
+                sk_SSL_CIPHER_free(skCipherOld);
             return FORBIDDEN;
         }
-        /* determine whether the cipher suite was actually changed */
+        /* determine whether a renegotiation has to be forced */
         skCipher = SSL_get_ciphers(ssl);
-        if ((skCipherOld == NULL && skCipher != NULL) ||
-            (skCipherOld != NULL && skCipher == NULL)   )
-            renegotiate = TRUE;
-        else if (skCipherOld != NULL && skCipher != NULL) {
-            for (n = 0; n < sk_SSL_CIPHER_num(skCipher); n++) {
-                if (sk_SSL_CIPHER_find(skCipherOld, sk_SSL_CIPHER_value(skCipher, n)) < 0) {
-                    renegotiate = TRUE;
-                    break;
-                }
+        if (dc->nOptions & SSL_OPT_OPTRENEGOTIATE) {
+            /* optimized way */
+            if ((pCipher == NULL && skCipher != NULL) ||
+                (pCipher != NULL && skCipher == NULL)   )
+                renegotiate = TRUE;
+            else if (pCipher != NULL && skCipher != NULL
+                     && sk_SSL_CIPHER_find(skCipher, pCipher) < 0) {
+                renegotiate = TRUE;
             }
-            for (n = 0; n < sk_SSL_CIPHER_num(skCipherOld); n++) {
-                if (sk_SSL_CIPHER_find(skCipher, sk_SSL_CIPHER_value(skCipherOld, n)) < 0) {
-                    renegotiate = TRUE;
-                    break;
+        }
+        else {
+            /* paranoid way */
+            if ((skCipherOld == NULL && skCipher != NULL) ||
+                (skCipherOld != NULL && skCipher == NULL)   )
+                renegotiate = TRUE;
+            else if (skCipherOld != NULL && skCipher != NULL) {
+                for (n = 0; !renegotiate && n < sk_SSL_CIPHER_num(skCipher); n++) {
+                    if (sk_SSL_CIPHER_find(skCipherOld, sk_SSL_CIPHER_value(skCipher, n)) < 0)
+                        renegotiate = TRUE;
+                }
+                for (n = 0; !renegotiate && n < sk_SSL_CIPHER_num(skCipherOld); n++) {
+                    if (sk_SSL_CIPHER_find(skCipher, sk_SSL_CIPHER_value(skCipherOld, n)) < 0)
+                        renegotiate = TRUE;
                 }
             }
         }
-        /* free old cipher suite */
+        /* cleanup */
         if (skCipherOld != NULL)
             sk_SSL_CIPHER_free(skCipherOld);
+        /* tracing */
+        if (renegotiate)
+            ssl_log(r->server, SSL_LOG_TRACE,
+                    "Reconfigured cipher suite will force renegotiation");
     }
 
     /*
-     * override of SSLVerifyDepth:
-     * This is handled by us manually inside the verify callback
-     * function and not by OpenSSL internally. And our function is
-     * aware of both the per-server and per-directory contexts.
-     * All we have to do is to force the renegotiation when the
-     * maximum allowed depth is changed.
+     * override of SSLVerifyDepth
+     *
+     * The depth checks are handled by us manually inside the verify callback
+     * function and not by OpenSSL internally (and our function is aware of
+     * both the per-server and per-directory contexts). So we cannot ask
+     * OpenSSL about the currently verify depth. Instead we remember it in our
+     * ap_ctx attached to the SSL* of OpenSSL.  We've to force the
+     * renegotiation if the reconfigured/new verify depth is less than the
+     * currently active/remembered verify depth (because this means more
+     * restriction on the certificate chain).
      */
     if (dc->nVerifyDepth != UNSET) {
-        if (dc->nVerifyDepth != sc->nVerifyDepth)
+        apctx = SSL_get_app_data2(ssl);
+        if ((vp = ap_ctx_get(apctx, "ssl::verify::depth")) != NULL)
+            n = AP_CTX_PTR2NUM(vp);
+        else
+            n = sc->nVerifyDepth;
+        ap_ctx_set(apctx, "ssl::verify::depth",
+                   AP_CTX_NUM2PTR(dc->nVerifyDepth));
+        /* determine whether a renegotiation has to be forced */
+        if (dc->nVerifyDepth < n) {
             renegotiate = TRUE;
+            ssl_log(r->server, SSL_LOG_TRACE,
+                    "Reduced client verification depth will force renegotiation");
+        }
     }
 
-    /* 
+    /*
      * override of SSLVerifyClient
+     *
+     * We force a renegotiation if the reconfigured/new verify type is
+     * stronger than the currently active verify type. 
+     *
+     * The order is: none << optional_no_ca << optional << require
+     *
+     * Additionally the following optimization is possible here: When the
+     * currently active verify type is "none" but a client certificate is
+     * already known/present, it's enough to manually force a client
+     * verification but at least skip the I/O-intensive renegotation
+     * handshake.
      */
     if (dc->nVerifyClient != SSL_CVERIFY_UNSET) {
-        /* remember old verify mode */
+        /* remember old state */
         nVerifyOld = SSL_get_verify_mode(ssl);
-        /* configure new verify mode */
+        /* configure new state */
         nVerify = SSL_VERIFY_NONE;
         if (dc->nVerifyClient == SSL_CVERIFY_REQUIRE)
             nVerify |= SSL_VERIFY_PEER|SSL_VERIFY_FAIL_IF_NO_PEER_CERT;
@@ -776,9 +863,25 @@ int ssl_hook_Access(request_rec *r)
             nVerify |= SSL_VERIFY_PEER;
         SSL_set_verify(ssl, nVerify, ssl_callback_SSLVerify);
         SSL_set_verify_result(ssl, X509_V_OK);
-        /* determine whether the verify mode was actually changed */
-        if (nVerify != nVerifyOld) 
-            renegotiate = TRUE;
+        /* determine whether we've to force a renegotiation */
+        if (nVerify != nVerifyOld) {
+            if (   (   (nVerifyOld == SSL_VERIFY_NONE)
+                    && (nVerify    != SSL_VERIFY_NONE))
+                || (  !(nVerifyOld &  SSL_VERIFY_PEER)
+                    && (nVerify    &  SSL_VERIFY_PEER))
+                || (  !(nVerifyOld &  (SSL_VERIFY_PEER|SSL_VERIFY_FAIL_IF_NO_PEER_CERT))
+                    && (nVerify    &  (SSL_VERIFY_PEER|SSL_VERIFY_FAIL_IF_NO_PEER_CERT)))) {
+                renegotiate = TRUE;
+                /* optimization */
+                if (   dc->nOptions & SSL_OPT_OPTRENEGOTIATE
+                    && nVerifyOld == SSL_VERIFY_NONE
+                    && SSL_get_peer_certificate(ssl) != NULL)
+                    renegotiate_quick = TRUE;
+                ssl_log(r->server, SSL_LOG_TRACE,
+                        "Changed client verification type will force %srenegotiation",
+                        renegotiate_quick ? "quick " : "");
+             }
+        }
     }
 
     /*
@@ -792,13 +895,13 @@ int ssl_hook_Access(request_rec *r)
      *  chance to provide this functionality...
      */
 #ifdef SSL_EXPERIMENTAL
-    if (   (   dc->szCACertificateFile != NULL 
-            && (   sc->szCACertificateFile == NULL 
-                || (   sc->szCACertificateFile != NULL 
+    if (   (   dc->szCACertificateFile != NULL
+            && (   sc->szCACertificateFile == NULL
+                || (   sc->szCACertificateFile != NULL
                     && strNE(dc->szCACertificateFile, sc->szCACertificateFile))))
-        || (   dc->szCACertificatePath != NULL 
-            && (   sc->szCACertificatePath == NULL 
-                || (   sc->szCACertificatePath != NULL 
+        || (   dc->szCACertificatePath != NULL
+            && (   sc->szCACertificatePath == NULL
+                || (   sc->szCACertificatePath != NULL
                     && strNE(dc->szCACertificatePath, sc->szCACertificatePath)))) ) {
         cpCAFile = dc->szCACertificateFile != NULL ?
                    dc->szCACertificateFile : sc->szCACertificateFile;
@@ -806,7 +909,7 @@ int ssl_hook_Access(request_rec *r)
                    dc->szCACertificatePath : sc->szCACertificatePath;
         /*
            FIXME: This should be...
-           if (!SSL_load_verify_locations(ssl, cpCAFile, cpCAPath)) { 
+           if (!SSL_load_verify_locations(ssl, cpCAFile, cpCAPath)) {
            ...but OpenSSL still doesn't provide this!
          */
         if (!SSL_CTX_load_verify_locations(ctx, cpCAFile, cpCAPath)) {
@@ -815,7 +918,7 @@ int ssl_hook_Access(request_rec *r)
                     "for client authentication");
             return FORBIDDEN;
         }
-        if ((skCAList = ssl_init_FindCAList(r->server, r->pool, 
+        if ((skCAList = ssl_init_FindCAList(r->server, r->pool,
                                             cpCAFile, cpCAPath)) == NULL) {
             ssl_log(r->server, SSL_LOG_ERROR,
                     "Unable to determine list of available "
@@ -825,14 +928,16 @@ int ssl_hook_Access(request_rec *r)
         SSL_set_client_CA_list(ssl, skCAList);
         renegotiate = TRUE;
         reconfigured_locations = TRUE;
+        ssl_log(r->server, SSL_LOG_TRACE,
+                "Changed client verification locations will force renegotiation");
     }
 #endif /* SSL_EXPERIMENTAL */
 
-    /* 
+    /*
      * now do the renegotiation if anything was actually reconfigured
      */
     if (renegotiate) {
-        /* 
+        /*
          * Now we force the SSL renegotation by sending the Hello Request
          * message to the client. Here we have to do a workaround: Actually
          * OpenSSL returns immediately after sending the Hello Request (the
@@ -844,33 +949,70 @@ int ssl_hook_Access(request_rec *r)
          * state explicitly and continue the handshake manually.
          */
         ssl_log(r->server, SSL_LOG_INFO, "Requesting connection re-negotiation");
-        SSL_renegotiate(ssl);
-        SSL_do_handshake(ssl);
-        if (SSL_get_state(ssl) != SSL_ST_OK) {
-            ssl_log(r->server, SSL_LOG_ERROR, "Re-negotation request failed");
-            return FORBIDDEN;
+        if (renegotiate_quick) {
+            /* perform just a manual re-verification of the peer */
+            ssl_log(r->server, SSL_LOG_TRACE,
+                    "Performing quick renegotiation: just re-verifying the peer");
+            certstore = SSL_CTX_get_cert_store(ctx);
+            if (certstore == NULL) {
+                ssl_log(r->server, SSL_LOG_ERROR, "Cannot find certificate storage");
+                return FORBIDDEN;
+            }
+            certstack = SSL_get_peer_cert_chain(ssl);
+            if (certstack == NULL || sk_X509_num(certstack) == 0) {
+                ssl_log(r->server, SSL_LOG_ERROR, "Cannot find peer certificate chain");
+                return FORBIDDEN;
+            }
+            cert = sk_X509_value(certstack, 0);
+            X509_STORE_CTX_init(&certstorectx, certstore, cert, certstack);
+            depth = SSL_get_verify_depth(ssl);
+            if (depth >= 0)
+                X509_STORE_CTX_set_depth(&certstorectx, depth);
+            X509_STORE_CTX_set_ex_data(&certstorectx,
+                SSL_get_ex_data_X509_STORE_CTX_idx(), (char *)ssl);
+            if (!X509_verify_cert(&certstorectx))
+                ssl_log(r->server, SSL_LOG_ERROR|SSL_ADD_SSLERR, 
+                        "Re-negotiation verification step failed");
+            SSL_set_verify_result(ssl, certstorectx.error);
+            X509_STORE_CTX_cleanup(&certstorectx);
         }
-        ssl_log(r->server, SSL_LOG_INFO, "Awaiting re-negotiation handshake");
-        SSL_set_state(ssl, SSL_ST_ACCEPT);
-        SSL_do_handshake(ssl);
-        if (SSL_get_state(ssl) != SSL_ST_OK) {
-            ssl_log(r->server, SSL_LOG_ERROR, 
-                    "Re-negotiation handshake failed: Not accepted by client!?");
-            return FORBIDDEN;
+        else {
+            /* do a full renegotiation */
+            ssl_log(r->server, SSL_LOG_TRACE,
+                    "Performing full renegotiation: complete handshake protocol");
+            if (r->main != NULL)
+                SSL_set_session_id_context(ssl, (unsigned char *)&(r->main), sizeof(r->main));
+            else
+                SSL_set_session_id_context(ssl, (unsigned char *)&r, sizeof(r));
+            SSL_renegotiate(ssl);
+            SSL_do_handshake(ssl);
+            if (SSL_get_state(ssl) != SSL_ST_OK) {
+                ssl_log(r->server, SSL_LOG_ERROR, "Re-negotiation request failed");
+                return FORBIDDEN;
+            }
+            ssl_log(r->server, SSL_LOG_INFO, "Awaiting re-negotiation handshake");
+            SSL_set_state(ssl, SSL_ST_ACCEPT);
+            SSL_do_handshake(ssl);
+            if (SSL_get_state(ssl) != SSL_ST_OK) {
+                ssl_log(r->server, SSL_LOG_ERROR,
+                        "Re-negotiation handshake failed: Not accepted by client!?");
+                return FORBIDDEN;
+            }
         }
 
         /*
          * Finally check for acceptable renegotiation results
          */
         if (dc->nVerifyClient != SSL_CVERIFY_NONE) {
-            if (SSL_get_verify_result(ssl) != X509_V_OK) {
-                ssl_log(r->server, SSL_LOG_ERROR, 
+            if (   dc->nVerifyClient == SSL_CVERIFY_REQUIRE
+                && SSL_get_verify_result(ssl) != X509_V_OK  ) {
+                ssl_log(r->server, SSL_LOG_ERROR,
                         "Re-negotiation handshake failed: Client verification failed");
                 return FORBIDDEN;
             }
-            if (   dc->nVerifyClient == SSL_CVERIFY_REQUIRE 
+            if (   dc->nVerifyClient == SSL_CVERIFY_REQUIRE
                 && SSL_get_peer_certificate(ssl) == NULL   ) {
-                ssl_log(r->server, SSL_LOG_ERROR, 
+                ssl_log(r->server, SSL_LOG_ERROR,
                         "Re-negotiation handshake failed: Client certificate missing");
                 return FORBIDDEN;
             }
@@ -991,6 +1133,8 @@ int ssl_hook_Auth(request_rec *r)
     ssl_util_uuencode(b2, b1, FALSE);
     ap_snprintf(b1, sizeof(b1), "Basic %s", b2);
     ap_table_set(r->headers_in, "Authorization", b1);
+    ssl_log(r->server, SSL_LOG_INFO,
+            "Faking HTTP Basic Auth header: \"Authorization: %s\"", b1);
 
     return DECLINED;
 }
@@ -1220,19 +1364,22 @@ int ssl_callback_SSLVerify(int ok, X509_STORE_CTX *ctx)
     request_rec *r;
     SSLSrvConfigRec *sc;
     SSLDirConfigRec *dc;
+    ap_ctx *actx;
     X509 *xs;
     int errnum;
     int errdepth;
     char *cp;
     char *cp2;
     int depth;
+    int verify;
 
     /*
      * Get Apache context back through OpenSSL context
      */
     ssl  = (SSL *)X509_STORE_CTX_get_app_data(ctx);
     conn = (conn_rec *)SSL_get_app_data(ssl);
-    r    = (request_rec *)SSL_get_app_data2(ssl);
+    actx = (ap_ctx *)SSL_get_app_data2(ssl);
+    r    = (request_rec *)ap_ctx_get(actx, "ssl::request_rec");
     s    = conn->server;
     sc   = mySrvConfig(s);
     dc   = (r != NULL ? myDirConfig(r) : NULL);
@@ -1261,11 +1408,15 @@ int ssl_callback_SSLVerify(int ok, X509_STORE_CTX *ctx)
     /*
      * Check for optionally acceptable non-verifiable issuer situation
      */
+    if (dc != NULL && dc->nVerifyClient != SSL_CVERIFY_UNSET)
+        verify = dc->nVerifyClient;
+    else
+        verify = sc->nVerifyClient;
     if (   (   errnum == X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT
             || errnum == X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN
             || errnum == X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY
             || errnum == X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE  )
-        && sc->nVerifyClient == SSL_CVERIFY_OPTIONAL_NO_CA            ) {
+        && verify == SSL_CVERIFY_OPTIONAL_NO_CA                       ) {
         ssl_log(s, SSL_LOG_TRACE,
                 "Certificate Verification: Verifiable Issuer is configured as "
                 "optional, therefore we're accepting the certificate");
@@ -1437,8 +1588,8 @@ int ssl_callback_SSLVerify_CRL(
     }
 
     /*
-     * Try to retrieve a CRL corresponding to the _issuer of
-     * the current certificate in order to check for revokation.
+     * Try to retrieve a CRL corresponding to the _issuer_ of
+     * the current certificate in order to check for revocation.
      */
     memset((char *)&obj, 0, sizeof(obj));
     rc = SSL_X509_STORE_lookup(sc->pRevocationStore, X509_LU_CRL, issuer, &obj);
